/* Copyright (c) 2012 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 *
 * @defgroup ble_sdk_srv_nus Alford Current Time Service
 * @{
 * @ingroup ble_sdk_srv
 * @brief Alford Current Time Service module.
 *
 * @details This module implements the Alford Current Time Service with an RX and TX characteristics. Data
 *          received will be passed to the application, while data sent will be transferred as
 *          notifications.
 *          During initialization it adds the Alford Current Time Service and its characteristics to
 *          the BLE stack datanuse.
 *
 * @note The application must propagate BLE stack events to the Alford Current Time Service module by calling
 *       ble_cts_on_ble_evt() from the from the @ref ble_stack_handler callback.
 */

#ifndef BLE_CTS_ALFORD_H__
#define BLE_CTS_ALFORD_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_srv_common.h"


#define BLE_UUID_CTS_ALFORD_SERVICE 0x1600
#define BLE_UUID_CTS_ALARM_CHARACTERISTIC 0x1601
#define BLE_UUID_CTS_CURRENT_CHARACTERISTIC 0x1602

/**@brief Current Time Service event type. */
typedef enum
{
    BLE_CTS_EVT_CURRENT_TIME_CONFIG,
    BLE_CTS_EVT_ALARM_CLOCK_CONFIG,

    BLE_CTS_EVT_MAX
} ble_cts_evt_type_t;

/**@brief Current Time Service event. */
typedef struct
{
    ble_cts_evt_type_t cts_evt_type;                                  /**< Type of event. */
} ble_cts_evt_t;

// Forward declaration of the ble_cts_t type.
typedef struct ble_cts_s ble_cts_t;

/**@brief Alford Current Time Service event handler type. */
typedef void (*ble_cts_evt_handler_t) (ble_cts_t * p_cts, ble_cts_evt_t *cts_evt, uint8_t * data, uint16_t length);

/**@brief Alford Current Time Service init structure. This contains all options and data needed for
 *        initialization of the service.*/
typedef struct
{
	  uint8_t                  uuid_type;               /**< UUID type for Alford Current Time Service Base UUID. */
    ble_cts_evt_handler_t evt_handler;              /**< Event handler to be called for handling received data. */
} ble_cts_init_t;

/**@brief Alford Current Time Service structure. This contains various status information for the service. */
typedef struct ble_cts_s
{
    uint8_t                  uuid_type;               /**< UUID type for Alford Current Time Service Base UUID. */
    uint16_t
    service_handle;          /**< Handle of Alford Current Time Service (as provided by the BLE stack). */
    ble_gatts_char_handles_t current_time_handles;              /**< Handles related to the TX characteristic. */
    ble_gatts_char_handles_t alarm_handles;              /**< Handles related to the RX characteristic. */
    uint16_t
    conn_handle;             /**< Handle of the current connection (as provided by the BLE stack, is BLE_CONN_HANDLE_INVALID if not in a connection). */
    bool                     is_notification_enabled;
    ble_cts_evt_handler_t   evt_handler;            /**< Event handler to be called for handling received data. */
} ble_cts_t;

/**@brief Initialize the Alford Current Time Service.
 *
 * @param[out]  p_cts       Alford Current Time Service structure. This structure will have to be supplied by
 *                          the application. It will be initialized by this function, and will later
 *                          be used to identify this particular service instance.
 * @param[in]   p_cts_init  Information needed to initialize the service.
 *
 * @return      NRF_SUCCESS on successful initialization of service, otherwise an error code.
 */
uint32_t ble_cst_init(ble_cts_t * p_cts, const ble_cts_init_t * p_cts_init);

/**@brief Alford Current Time Service BLE stack event handler.
 *
 * @details Handles all events from the BLE stack of interest to the Alford Current Time Service.
 *
 * @note For the requirements in the nus specification to be fulfilled,
 *       ble_nus_battery_level_update() must be called upon reconnection if the
 *       battery level has changed while the service has been disconnected from a bonded
 *       client.
 *
 * @param[in]   p_cts      Alford Current Time Service structure.
 * @param[in]   p_ble_evt  Event received from the BLE stack.
 */
void ble_cts_on_ble_evt(ble_cts_t * p_cts, ble_evt_t * p_ble_evt);

/**@brief Send string.
 *
 * @details The application calls this function to send a string to the peer device.
 *
 *
 * @param[in]   p_cts          Alford Current Time Service structure.
 * @param[in]   string         String to be sent.
 * @param[in]   length         Length of string.
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */
uint32_t ble_cts_send_string(ble_cts_t * p_cts, uint8_t * string, uint16_t length);

#endif // BLE_CTS_ALFORD_H__

/** @} */
